@@ -12,6 +12,8 @@ import obsws_python as obs
 from GameSentenceMiner.util import configuration
 from GameSentenceMiner.util.configuration import *
 from GameSentenceMiner.util.gsm_utils import sanitize_filename, make_unique_file_name
+import tkinter as tk
+from tkinter import messagebox
 
 client: obs.ReqClient = None
 event_client: obs.EventClient = None
@@ -22,14 +24,18 @@ logging.getLogger("obsws_python").setLevel(logging.CRITICAL)
 connecting = False
 
 class OBSConnectionManager(threading.Thread):
-    def __init__(self):
+    def __init__(self, check_output=True):
         super().__init__()
         self.daemon = True
         self.running = True
+        self.check_connection_interval = 1
+        self.said_no_to_replay_buffer = False
+        self.counter = 0
+        self.check_output = check_output
 
     def run(self):
         while self.running:
-            time.sleep(1)
+            time.sleep(self.check_connection_interval)
             try:
                 if not connecting:
                     client.get_version()
@@ -37,9 +43,42 @@ class OBSConnectionManager(threading.Thread):
                 logger.info(f"OBS WebSocket not connected. Attempting to reconnect... {e}")
                 gsm_status.obs_connected = False
                 asyncio.run(connect_to_obs())
+            if self.counter % 5 == 0 and not get_config().obs.turn_off_output_check and self.check_output:
+                replay_buffer_status = get_replay_buffer_status()
+                if replay_buffer_status and self.said_no_to_replay_buffer:
+                    self.said_no_to_replay_buffer = False
+                    self.counter = 0
+                if gsm_status.obs_connected and not replay_buffer_status and not self.said_no_to_replay_buffer:
+                    try:
+                        self.check_output()
+                    except Exception as e:
+                        pass
+            self.counter += 1
 
     def stop(self):
         self.running = False
+    
+    def check_output(self):
+        img = get_screenshot_PIL(compression=100, img_format='jpg', width=1280, height=720)
+        extrema = img.getextrema()
+        if isinstance(extrema[0], tuple):
+            is_empty = all(e[0] == e[1] for e in extrema)
+        else:
+            is_empty = extrema[0] == extrema[1]
+        if is_empty:
+            return
+        else:
+            root = tk.Tk()
+            root.attributes('-topmost', True)
+            root.withdraw()
+            root.deiconify()
+            result = messagebox.askyesno("GSM - Replay Buffer", "The replay buffer is not running, but there seems to be output in OBS. Do you want to start it? (If you click 'No', you won't be asked until you either restart GSM or start/stop replay buffer manually.)")
+            root.destroy()
+            if not result:
+                self.said_no_to_replay_buffer = True
+                self.counter = 0
+                return
+            start_replay_buffer()
 
 def get_obs_path():
     return os.path.join(configuration.get_app_directory(), 'obs-studio/bin/64bit/obs64.exe')
@@ -137,7 +176,7 @@ def get_obs_websocket_config_values():
         full_config.save()
         reload_config()
 
-async def connect_to_obs(retry=5):
+async def connect_to_obs(retry=5, check_output=True):
     global client, obs_connection_manager, event_client, connecting
     if is_windows():
         get_obs_websocket_config_values()
@@ -160,7 +199,7 @@ async def connect_to_obs(retry=5):
             gsm_status.obs_connected = True
             logger.info("Connected to OBS WebSocket.")
             if not obs_connection_manager:
-                obs_connection_manager = OBSConnectionManager()
+                obs_connection_manager = OBSConnectionManager(check_output=check_output)
                 obs_connection_manager.start()
             update_current_game()
             break  # Exit the loop once connected
@@ -176,7 +215,7 @@ async def connect_to_obs(retry=5):
             retry -= 1
     connecting = False
 
-def connect_to_obs_sync(retry=2):
+def connect_to_obs_sync(retry=2, check_output=True):
     global client, obs_connection_manager, event_client
     if is_windows():
         get_obs_websocket_config_values()
@@ -196,7 +235,7 @@ def connect_to_obs_sync(retry=2):
                 timeout=1,
             )
             if not obs_connection_manager:
-                obs_connection_manager = OBSConnectionManager()
+                obs_connection_manager = OBSConnectionManager(check_output=check_output)
                 obs_connection_manager.start()
             update_current_game()
             logger.info("Connected to OBS WebSocket.")
@@ -248,9 +287,9 @@ def toggle_replay_buffer():
 
 def start_replay_buffer():
     try:
-        status = get_replay_buffer_status()
-        if status:
-            client.start_replay_buffer()
+        response = client.start_replay_buffer()
+        if response and response.ok:
+            logger.info("Replay buffer started.")
     except Exception as e:
         logger.error(f"Error starting replay buffer: {e}")
 
@@ -258,12 +297,14 @@ def get_replay_buffer_status():
     try:
         return client.get_replay_buffer_status().output_active
     except Exception as e:
-        logger.warning(f"Error getting replay buffer status: {e}")
+        logger.debug(f"Error getting replay buffer status: {e}")
         return None
 
 def stop_replay_buffer():
     try:
-        client.stop_replay_buffer()
+        response = client.stop_replay_buffer()
+        if response and response.ok:
+            logger.info("Replay buffer stopped.")
     except Exception as e:
         logger.warning(f"Error stopping replay buffer: {e}")
 
