@@ -15,6 +15,7 @@ import numpy as np
 import rapidfuzz.fuzz
 from PIL import Image
 from loguru import logger
+import regex
 import requests
 
 
@@ -71,7 +72,12 @@ except ImportError:
     pass
 
 try:
-    import oneocr
+    try:
+        if os.path.exists(os.path.expanduser('~/.config/oneocr/oneocr.dll')):
+            import oneocr
+    except Exception as e:
+        oneocr = None
+        logger.warning(f'Failed to import OneOCR: {e}', exc_info=True)
 except ImportError:
     pass
 
@@ -271,8 +277,10 @@ class GoogleLens:
     available = False
 
     def __init__(self, lang='ja'):
+        import regex
         self.regex = get_regex(lang)
         self.initial_lang = lang
+        self.punctuation_regex = regex.compile(r'[\p{P}\p{S}]')
         if 'betterproto' not in sys.modules:
             logger.warning('betterproto not available, Google Lens will not work!')
         else:
@@ -370,6 +378,9 @@ class GoogleLens:
                 for line in paragraph['lines']:
                     if furigana_filter_sensitivity:
                         for word in line['words']:
+                            if self.punctuation_regex.findall(word['plain_text']):
+                                res += word['plain_text'] + word['text_separator']
+                                continue
                             if 'geometry' not in word:
                                 res += word['plain_text'] + word['text_separator']
                                 continue
@@ -378,7 +389,7 @@ class GoogleLens:
                             if word_width > furigana_filter_sensitivity and word_height > furigana_filter_sensitivity:
                                 res += word['plain_text'] + word['text_separator']
                             else:
-                                skipped.extend([word['plain_text'] for word in line['words']])
+                                skipped.extend(word['plain_text'])
                                 continue
                     else:
                         for word in line['words']:
@@ -434,6 +445,7 @@ class GoogleLens:
 
         if skipped:
             logger.info(f"Skipped {len(skipped)} chars due to furigana filter sensitivity: {furigana_filter_sensitivity}")
+            logger.debug(f"Skipped chars: {''.join(skipped)}")
 
         # img.close()
         return x
@@ -859,15 +871,20 @@ class OneOCR:
     available = False
 
     def __init__(self, config={}, lang='ja'):
+        import regex
         self.initial_lang = lang
         self.regex = get_regex(lang)
+        self.punctuation_regex = regex.compile(r'[\p{P}\p{S}]')
         if sys.platform == 'win32':
             if int(platform.release()) < 10:
                 logger.warning('OneOCR is not supported on Windows older than 10!')
             elif 'oneocr' not in sys.modules:
                 logger.warning('oneocr not available, OneOCR will not work!')
+            elif not os.path.exists(os.path.expanduser('~/.config/oneocr/oneocr.dll')):
+                logger.warning('OneOCR DLLs not found, please install OwOCR Dependencies via OCR Tab in GSM.')
             else:
                 try:
+                    logger.info(f'Loading OneOCR model')
                     self.model = oneocr.OcrEngine()
                 except RuntimeError as e:
                     logger.warning(e + ', OneOCR will not work!')
@@ -942,17 +959,21 @@ class OneOCR:
                 boxes = []
                 if furigana_filter_sensitivity > 0:
                     for line in filtered_lines:
-                        x1, x2, x3, x4 = line['bounding_rect']['x1'], line['bounding_rect']['x2'], \
-                            line['bounding_rect']['x3'], line['bounding_rect']['x4']
-                        y1, y2, y3, y4 = line['bounding_rect']['y1'], line['bounding_rect']['y2'], \
-                            line['bounding_rect']['y3'], line['bounding_rect']['y4']
-                        width = max(x2 - x1, x3 - x4)
-                        height = max(y3 - y1, y4 - y2)
-                        if width > furigana_filter_sensitivity and height > furigana_filter_sensitivity:
-                            res += line['text']
-                        else:
-                            skipped.extend(char for char in line['text'])
-                            continue
+                        for char in line['words']:
+                            if self.punctuation_regex.findall(char['text']):
+                                res += char['text']
+                                continue
+                            x1, x2, x3, x4 = char['bounding_rect']['x1'], char['bounding_rect']['x2'], \
+                            char['bounding_rect']['x3'], char['bounding_rect']['x4']
+                            y1, y2, y3, y4 = char['bounding_rect']['y1'], char['bounding_rect']['y2'], \
+                                char['bounding_rect']['y3'], char['bounding_rect']['y4']
+                            width = max(x2 - x1, x3 - x4)
+                            height = max(y3 - y1, y4 - y2)
+                            if width > furigana_filter_sensitivity and height > furigana_filter_sensitivity:
+                                res += char['text']
+                            else:
+                                skipped.extend(char for char in line['text'])
+                                continue
                         res += '\n'
                     # logger.info(
                     #     f"Skipped {len(skipped)} chars due to furigana filter sensitivity: {furigana_filter_sensitivity}")
@@ -1393,6 +1414,11 @@ class localLLMOCR:
 
     def __init__(self, config={}, lang='ja'):
         self.keep_llm_hot_thread = None
+        # All three config values are required: url, model, api_key
+        if not config or not (config.get('url') and config.get('model') and config.get('api_key')):
+            logger.warning('Local LLM OCR requires url, model, and api_key in config, Local LLM OCR will not work!')
+            return
+
         try:
             import openai
         except ImportError:
@@ -1400,16 +1426,20 @@ class localLLMOCR:
             return
         import openai, threading
         try:
-            self.api_url = config.get('api_url', 'http://localhost:1234/v1/chat/completions')
+            self.api_url = config.get('url', 'http://localhost:1234/v1/chat/completions')
             self.model = config.get('model', 'qwen2.5-vl-3b-instruct')
             self.api_key = config.get('api_key', 'lm-studio')
             self.keep_warm = config.get('keep_warm', True)
             self.custom_prompt = config.get('prompt', None)
             self.available = True
+            # if any(x in self.api_url for x in ['localhost', '127.0.0.1']):
+            #     if not self.check_connection(self.api_url):
+            #         logger.warning('Local LLM OCR API is not reachable')
+            #         return
             self.client = openai.OpenAI(
-                    base_url=self.api_url.replace('/v1/chat/completions', '/v1'),
-                    api_key=self.api_key
-                )
+                base_url=self.api_url.replace('/v1/chat/completions', '/v1'),
+                api_key=self.api_key
+            )
             if self.client.models.retrieve(self.model):
                 self.model = self.model
             logger.info(f'Local LLM OCR (OpenAI-compatible) ready with model {self.model}')
@@ -1418,6 +1448,24 @@ class localLLMOCR:
                 self.keep_llm_hot_thread.start()
         except Exception as e:
             logger.warning(f'Error initializing Local LLM OCR, Local LLM OCR will not work!')
+            
+    def check_connection(self, url, port=None):
+        import http.client
+        conn = http.client.HTTPConnection(url, port or 1234, timeout=0.1)
+        try:
+            conn.request("GET", "/v1/models")
+            response = conn.getresponse()
+            if response.status == 200:
+                logger.info('Local LLM OCR API is reachable')
+                return True
+            else:
+                logger.warning('Local LLM OCR API is not reachable')
+                return False
+        except Exception as e:
+            logger.warning(f'Error connecting to Local LLM OCR API: {e}')
+            return False
+        finally:
+            conn.close()
         
     def keep_llm_warm(self):
         def ocr_blank_black_image():
