@@ -17,7 +17,6 @@ from dataclasses_json import dataclass_json
 
 from importlib import metadata
 
-import requests
 
 
 OFF = 'OFF'
@@ -361,6 +360,7 @@ def get_current_version():
 
 def get_latest_version():
     try:
+        import requests
         response = requests.get(f"https://pypi.org/pypi/{PACKAGE_NAME}/json")
         latest_version = response.json()["info"]["version"]
         return latest_version
@@ -488,6 +488,10 @@ class Screenshot:
             self.screenshot_timing_setting = 'middle'
         if not self.screenshot_timing_setting and not self.use_beginning_of_line_as_screenshot and not self.use_new_screenshot_logic:
             self.screenshot_timing_setting = 'end'
+        if self.width and self.height == 0:
+            self.height = -1
+        if self.width == 0 and self.height:
+            self.width = -1
 
 
 @dataclass_json
@@ -498,7 +502,7 @@ class Audio:
     beginning_offset: float = -0.5
     end_offset: float = 0.5
     pre_vad_end_offset: float = 0.0
-    ffmpeg_reencode_options: str = '-c:a libopus -f opus -af \"afade=t=in:d=0.10\"' if is_windows() else ''
+    ffmpeg_reencode_options: str = '-c:a {encoder} -f {format} -af \"afade=t=in:d=0.10\"' if is_windows() else ''
     ffmpeg_reencode_options_to_use: str = ''
     external_tool: str = ""
     anki_media_collection: str = ""
@@ -508,6 +512,8 @@ class Audio:
     def __post_init__(self):
         self.ffmpeg_reencode_options_to_use = self.ffmpeg_reencode_options.replace(
             "{format}", self.extension).replace("{encoder}", supported_formats.get(self.extension, ''))
+        if not self.anki_media_collection:
+            self.anki_media_collection = get_default_anki_media_collection_path()
         if self.anki_media_collection:
             self.anki_media_collection = os.path.normpath(
                 self.anki_media_collection)
@@ -525,6 +531,7 @@ class OBS:
     password: str = "your_password"
     get_game_from_scene: bool = True
     minimum_replay_size: int = 0
+    turn_off_output_check: bool = False
 
 
 @dataclass_json
